// Taschenrechner

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <avr/pgmspace.h>

extern "C"
{
#include "lcd.h"
#include "usart.h"
// #include "memcheck.h"
#include "rc5.h"
#include "avr_f64.h"
}

#define uint unsigned int

const uint8_t RC5CommandsForDigits[10]=
{
	0, 1, 2, 3, 4, 5, 6, 7, 8, 9
};

#define RC5_COMMAND_FOR_RESET 14
#define RC5_COMMAND_FOR_DECIMAL_POINT 28
#define RC5_COMMAND_FOR_PLUS 16
#define RC5_COMMAND_FOR_MINUS 17
#define RC5_COMMAND_FOR_MULT 18
#define RC5_COMMAND_FOR_DIV 19
#define RC5_COMMAND_FOR_RESULT 30

// Um exp, log, sin, cos, tan, arcsin, arccos, arctan verfgbar zu machen, mssen die
// entsprechenden #defines in der Datei avr_f64.h aktiviert werden (und der Flash des C ausreichen).

#define RC5_COMMAND_FOR_EXP 15
#define RC5_COMMAND_FOR_LOG 11

#define RC5_COMMAND_FOR_SIN 31
#define RC5_COMMAND_FOR_COS 32
#define RC5_COMMAND_FOR_TAN 33

#define RC5_COMMAND_FOR_ARCSIN 34
#define RC5_COMMAND_FOR_ARCCOS 35
#define RC5_COMMAND_FOR_ARCTAN 36

#ifdef F_WITH_sin
float64_t sinus(float64_t x)
{
	return f_sin(x);
}
#endif

#ifdef F_WITH_cos
float64_t cosinus(float64_t x)
{
	return f_cos(x);
}
#endif

#ifdef F_WITH_tan
float64_t tangens(float64_t x)
{
	return f_tan(x);
}
#endif

#ifdef F_WITH_arcsin
float64_t arcsinus(float64_t x)
{
	return f_arcsin(x);
}
#endif

#ifdef F_WITH_arccos
float64_t arccosinus(float64_t x)
{
	return f_arccos(x);
}
#endif

#ifdef F_WITH_arctan
float64_t arctangens(float64_t x)
{
	return f_arctan(x);
}
#endif

const struct
{
	uint8_t rc5_command;
	float64_t (*func)(float64_t x);
}
RC5CommandsForUnaryFunctions[]=
{	// Am Ende muss { 0, 0 } folgen !
#ifdef F_WITH_exp
	{ RC5_COMMAND_FOR_EXP, f_exp },
#endif
#ifdef F_WITH_log
	{ RC5_COMMAND_FOR_LOG, f_log },
#endif
#ifdef F_WITH_sin
	{ RC5_COMMAND_FOR_LOG, sinus },
#endif
#ifdef F_WITH_cos
	{ RC5_COMMAND_FOR_LOG, cosinus },
#endif
#ifdef F_WITH_tan
	{ RC5_COMMAND_FOR_LOG, tangens },
#endif
#ifdef F_WITH_arcsin
	{ RC5_COMMAND_FOR_LOG, arcsinus },
#endif
#ifdef F_WITH_arccos
	{ RC5_COMMAND_FOR_LOG, arccosinus },
#endif
#ifdef F_WITH_arctan
	{ RC5_COMMAND_FOR_LOG, arctangens },
#endif
	{ 0, 0 }
};

int main(void)
{
	lcd_init(LCD_DISP_ON);
	rc5_init();
	//usart_init(9600);

    lcd_clrscr();

    lcd_puts("Taschenrechner");
	while(!read_rc5())
		;

	float64_t z1, z2;
	static char s[2][33], op[2];
	static char buffer[33];
	uint8_t i;
	uint j;

	while(1)
	{
reset:
		bool z1_initialisiert=false, z2_initialisiert=false;
		s[0][0]=0;
		op[0]=0;

eingabe:
		lcd_clrscr();
		lcd_gotoxy(0,0);
		lcd_puts(0==s[0][0] ? "0" : s[0]);
		if(0!=op[0])
		{
			lcd_gotoxy(15,0);
			buffer[0]=op[0]; buffer[1]=0;
			lcd_puts(buffer);
		}

		for(uint8_t nop=0 ; nop<2 ; nop++)
		{
			bool komma=false, first=true, after_function_evaluation=false;
			char *sa=s[nop], *aop=&op[nop];
			uint n=strlen(sa);
			s[1][0]=0;
			op[1]=0;

			if(0==*aop || 0!=nop)
			while(1)
			{
				while(0==(j=read_rc5()) || RC5_IS_REPEAT(j))
					;
				i=RC5_KEY_CODE(j);
				if(RC5_COMMAND_FOR_RESET==i)
					goto reset;
				for(j=0 ; j<10 && RC5CommandsForDigits[j]!=i ; j++)
					;
				if(10!=j || (!komma && RC5_COMMAND_FOR_DECIMAL_POINT==i))
				{
					if(0==nop)
						z1_initialisiert=false;
					else
						z2_initialisiert=false;
					if((0==*aop && first && 0==nop) || after_function_evaluation)
					{
						sa[0]=0; n=0; lcd_clrscr();
						lcd_gotoxy(0,nop);
						lcd_puts("                ");
					}
					first=after_function_evaluation=false;
					if(n<15)
					{
						if(RC5_COMMAND_FOR_DECIMAL_POINT==i)
						{
							komma=true;
							if(0==sa[0] || ('0'==sa[0] && 0==sa[1]))
								sa[n++]='0';
						}
						if(0!=i || 0!=sa[0])
							sa[n++]=(RC5_COMMAND_FOR_DECIMAL_POINT==i) ? '.' : ('0'+j);
						sa[n]=0;
						lcd_gotoxy(0,nop);
						lcd_puts(0==sa[0] ? "0" : sa);
					}
				}
				bool is_op=false;
				if(RC5_COMMAND_FOR_PLUS==i)
					{ *aop='+'; is_op=true; }
				else if(RC5_COMMAND_FOR_MINUS==i)
					{ *aop='-'; is_op=true; }
				else if(RC5_COMMAND_FOR_MULT==i)
					{ *aop='*'; is_op=true; }
				else if(RC5_COMMAND_FOR_DIV==i)
					{ *aop='/'; is_op=true; }
				else if(RC5_COMMAND_FOR_RESULT==i)
					{ *aop='='; is_op=true; }

				if(is_op)
				{
					if(0!=nop && first)
					{
						op[0]=op[1];
						nop=0; // Es wurden zwei Operatoren (+, -, *, /) unmittelbar hintereinander gedrckt
						// ==> Den letzten Operator verwenden und auf Eingabe des zweiten Operanden warten.
					}
					if(0==nop) 
					{	// hier KEIN else if !
						if('='==*aop)
							{ op[0]=0; goto eingabe; }
						lcd_gotoxy(15,0);
						buffer[0]=*aop; buffer[1]=0;
						lcd_puts(buffer);
					}
					break;
				}
				
				for(j=0; RC5CommandsForUnaryFunctions[j].func && i!=RC5CommandsForUnaryFunctions[j].rc5_command; j++)
					;
				if(RC5CommandsForUnaryFunctions[j].func)
				{
					float64_t tmp=f_atof(sa);
					if(0==nop && z1_initialisiert)
						tmp=z1;
					if(1==nop && z2_initialisiert)
						tmp=z2;
					tmp=RC5CommandsForUnaryFunctions[j].func(tmp);
					if(0==nop)
					{
						z1=tmp;
						z1_initialisiert=true;
					}
					else
					{
						z2=tmp;
						z2_initialisiert=true;
					}

					strcpy(sa, f_to_string(tmp, 15, 3));
					//strcpy(sa, f_to_decimalExp(tmp, 8, 0,0));

					lcd_gotoxy(0,nop);
					lcd_puts("               ");
					lcd_gotoxy(0,nop);
					lcd_puts(0==sa[0] ? "0" : sa);
					after_function_evaluation=true;
				}
			}			
		}
			
		if(!z1_initialisiert)
			z1=f_atof(s[0]);
		if(!z2_initialisiert)
			z2=f_atof(s[1]);
		if(op[0]=='+')
			z1=f_add(z1, z2);
		else if(op[0]=='-')
			z1=f_sub(z1, z2);
		else if(op[0]=='*')
			z1=f_mult(z1, z2);
		else if(op[0]=='/')
			z1=f_div(z1, z2);
		
		strcpy(s[0], f_to_string(z1, 15, 3));
		//strcpy(s[0], f_to_decimalExp(z1, 8, 0,0));

		op[0]=(op[1]=='=') ? 0 : op[1];
		z1_initialisiert=true;
		z2_initialisiert=false;
		goto eingabe;
	}
}
