/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    RS232 Routinen
----------------------------------------------------------------------------*/

#include "usart.h"

volatile unsigned char buffercounter = 0;
char usart_rx_buffer[BUFFER_SIZE];
struct UsartStatus usart_status;


// Einstellungen beim Windows-Hyperterminal:
//		Baudrate: wie an usart_init() bergeben (9600 funktioniert)
//		Datenbits: 8
//		Paritt: Keine
//		Stoppbits: 2 (Standardmig ist bei Windows 1 eingestellt !!!)
//		Flusssteuerung: Hardware

//----------------------------------------------------------------------------
//Init serielle Schnittstelle
void usart_init(unsigned int baudrate) 
{ 
  	//Enable TXEN im Register UCR TX-Data Enable
	UCR =(1 << TXEN | 1 << RXEN | 1<< RXCIE);
	//Teiler wird gesetzt 
	UBRR=(SYSCLK / (baudrate * 16L) - 1);
}

//----------------------------------------------------------------------------
//Routine fr die Serielle Ausgabe eines Zeichens
void usart_write_char(char c)
{
	if (c == '\n')
		usart_write_char('\r');
	//Warten solange bis Zeichen gesendet wurde
	while(!(USR & (1<<UDRE)));
	//Ausgabe des Zeichens
	UDR = c;
	return;
}

//------------------------------------------------------------------------------
/*void usart_write_P (const char *Buffer,...)
{
	va_list ap;
	va_start (ap, Buffer);	
	
	int format_flag;
	char str_buffer[10];
	char str_null_buffer[10];
	char move = 0;
	char Base = 0;
	int tmp = 0;
	char by;
	char *ptr;
		
	//Ausgabe der Zeichen
    for(;;)
	{
		by = pgm_read_byte(Buffer++);
		if(by==0) break; // end of format string
            
		if (by == '%')
		{
            by = pgm_read_byte(Buffer++);
			if (isdigit(by)>0)
				{
                                 
 				str_null_buffer[0] = by;
				str_null_buffer[1] = '\0';
				move = atoi(str_null_buffer);
                by = pgm_read_byte(Buffer++);
				}

			switch (by)
				{
                case 's':
                    ptr = va_arg(ap,char *);
                    while(*ptr) { usart_write_char(*ptr++); }
                    break;
				case 'b':
					Base = 2;
					goto ConversionLoop;
				case 'c':
					//Int to char
					format_flag = va_arg(ap,int);
					usart_write_char (format_flag++);
					break;
				case 'i':
					Base = 10;
					goto ConversionLoop;
				case 'o':
					Base = 8;
					goto ConversionLoop;
				case 'x':
					Base = 16;
					//****************************
					ConversionLoop:
					//****************************
					itoa(va_arg(ap,int),str_buffer,Base);
					int b=0;
					while (str_buffer[b++] != 0){};
					b--;
					if (b<move)
						{
						move -=b;
						for (tmp = 0;tmp<move;tmp++)
							{
							str_null_buffer[tmp] = '0';
							}
						//tmp ++;
						str_null_buffer[tmp] = '\0';
						strcat(str_null_buffer,str_buffer);
						strcpy(str_buffer,str_null_buffer);
						}
					usart_write_str (str_buffer);
					move =0;
					break;
				}
			
			}	
		else
		{
			usart_write_char ( by );	
		}
	}
	va_end(ap);
}*/

//----------------------------------------------------------------------------
//Ausgabe eines Strings
void usart_write_str(char *str)
{
	while (*str)
	{
		usart_write_char(*str++);
	}
}

void usart_write_long(char *str, long n)
{
	usart_write_str(str);
	if(n<0)
		{ usart_write_char('-'); usart_write_ulong(0, -n); }
	else
		usart_write_ulong(0, n);
}

void usart_write_ulong(char *str, unsigned long n)
{
	if(0!=str)
		usart_write_str(str);
	char buffer[15];
	ultoa(n, buffer, 10);
	usart_write_str(buffer);
}

void usart_write_int(char *str, int n)
{
	usart_write_long(str, (long)n);
}

void usart_write_uint(char *str, unsigned int n)
{
	usart_write_ulong(str, (unsigned long)n);
}


//----------------------------------------------------------------------------
//Empfang eines Zeichens
SIGNAL (SIG_USART_RECV)
{
	unsigned char receive_char;
	receive_char = (UDR);

	if (usart_status.usart_ready)
	{
		usart_status.usart_rx_ovl = 1;
		return; 
	}

	if (receive_char == '\r' && (!(usart_rx_buffer[buffercounter-1] == '\\')))
	{
		usart_rx_buffer[buffercounter] = 0;
		buffercounter = 0;
		usart_status.usart_ready = 1;
		return;    
	}

	if (buffercounter < BUFFER_SIZE - 1)
	{
		usart_rx_buffer[buffercounter++] = receive_char;    
	}
return;
}
